import os
from typing import Any, Callable, Dict, Iterator, List

import pytest
from pyfakefs.fake_filesystem import FakeFilesystem

from gcs_migration.parsers import BlackPearlMapEntry


def _blackpearl_map_entry_line(entry: BlackPearlMapEntry) -> str:
    return " ".join(
        [
            entry.username,
            entry.access_id,
            entry.secret_key,
        ]
    )


@pytest.fixture
def blackpearl_map_entry1(sharing_local_user: str) -> BlackPearlMapEntry:
    return BlackPearlMapEntry(sharing_local_user, "access_key_id1", "secret_key1")


@pytest.fixture
def blackpearl_map_entry2() -> BlackPearlMapEntry:
    return BlackPearlMapEntry("user2", "access_key_id2", "secret_key2")


@pytest.fixture
def blackpearl_map_entries(
    blackpearl_map_entry1: BlackPearlMapEntry,
    blackpearl_map_entry2: BlackPearlMapEntry,
    user_without_mapfile: str,
    mock_users: Callable[[List[str]], Dict[str, Any]],
) -> Iterator[List[BlackPearlMapEntry]]:

    mock_users([blackpearl_map_entry1.username, blackpearl_map_entry2.username])
    yield [blackpearl_map_entry1, blackpearl_map_entry2]


@pytest.fixture
def blackpearl_access_id_filename() -> str:
    return "/etc/blackpearl/AccessIDFile"


@pytest.fixture
def blackpearl_access_id_file(
    fs: FakeFilesystem,
    blackpearl_access_id_filename: str,
    blackpearl_map_entries: List[BlackPearlMapEntry],
) -> Iterator[str]:
    fs.makedirs(os.path.dirname(blackpearl_access_id_filename), exist_ok=True)
    with open(blackpearl_access_id_filename, "w") as f:
        for entry in blackpearl_map_entries:
            print(_blackpearl_map_entry_line(entry), file=f)

    yield blackpearl_access_id_filename
