import os
from typing import Iterator

import pytest
from pyfakefs.fake_filesystem import FakeFilesystem

from gcs_migration.parsers import BlackPearlConfig


@pytest.fixture
def blackpearl_endpoint() -> str:
    return "https://blackpearl.example.org"


@pytest.fixture
def blackpearl_config(
    blackpearl_endpoint: str, blackpearl_access_id_filename: str
) -> str:
    return f"""\
EndPoint {blackpearl_endpoint}
AccessIDFile {blackpearl_access_id_filename}
"""


@pytest.fixture
def blackpearl_default_config_file_path() -> str:
    return "/etc/blackpearl/GridFTPConfig"


@pytest.fixture
def blackpearl_custom_config_file_path() -> str:
    return "/etc/blackpearl/gridftp-config"


@pytest.fixture
def blackpearl_default_config_file(
    fs: FakeFilesystem, blackpearl_default_config_file_path: str, blackpearl_config: str
) -> Iterator[str]:
    fs.makedirs(os.path.dirname(blackpearl_default_config_file_path))
    with open(blackpearl_default_config_file_path, "w") as f:
        f.write(blackpearl_config)
    yield blackpearl_default_config_file_path


@pytest.fixture
def blackpearl_custom_config_file(
    fs: FakeFilesystem, blackpearl_custom_config_file_path: str, blackpearl_config: str
) -> Iterator[str]:
    fs.makedirs(os.path.dirname(blackpearl_custom_config_file_path))
    with open(blackpearl_custom_config_file_path, "w") as f:
        f.write(blackpearl_config)
    yield blackpearl_custom_config_file_path


@pytest.fixture
def blackpearl_config_object(
    fs: FakeFilesystem, blackpearl_default_config_file: str
) -> BlackPearlConfig:
    return BlackPearlConfig(blackpearl_default_config_file)
